
type t = float array array

let read_file fname =
  let lines = ref [] in
  let ch = open_in fname in
  try
    while true do
      lines := input_line ch :: !lines
    done; !lines
  with End_of_file ->
    begin
      close_in ch;
      List.rev !lines
    end

let split_str sep str =
  Str.split (Str.regexp_string sep) str


let mat_list fname =
  let lines = read_file fname in
  List.map (fun line -> List.map (fun wd -> float_of_string wd) (List.filter (fun tk -> if tk <> "" then true else false) (split_str " " (line)))) lines

let mat_array mlist =
  let row = List.length mlist in
  let col = List.length (List.nth mlist 0) in
  Array.init row (fun i -> Array.init col (fun j -> List.nth (List.nth mlist j) i))

let kronecker i j = if i = j then 1. else 0.

open Lacaml.D
let ( *! ) a b =
  let a_mat = lacpy (Mat.of_array a) in
  let b_mat = lacpy (Mat.of_array b) in
  Mat.to_array (gemm a_mat b_mat)

let id_mat n = Array.init n (fun i -> Array.init n (kronecker i))

let () =
  let m = mat_list "mkb_mat_m" in
  print_int (List.length m);
  print_string " ";
  print_int (List.length (List.nth m 0));
  print_endline "";
  
  let b = mat_list "mkb_mat_b" in
  print_int (List.length b);
  print_string " ";
  print_int (List.length (List.nth b 0));
  print_endline "";

  let c = mat_list "mkb_mat_c" in
  print_int (List.length c);
  print_string " ";
  print_int (List.length (List.nth c 0));
  print_endline "";

  let d = mat_list "mkb_mat_d" in
  print_int (List.length d);
  print_string " ";
  print_int (List.length (List.nth d 0));
  print_endline "";

  let k = mat_list "mkb_mat_k" in
  print_int (List.length k);
  print_string " ";
  print_int (List.length (List.nth k 0));
  print_endline "";

  let pin = mat_list "mkb_mat_pi" in
  print_int (List.length pin);
  print_string " ";
  print_int (List.length (List.nth pin 0));
  print_endline "";

  let pou = mat_list "mkb_mat_po" in
  print_int (List.length pou);
  print_string " ";
  print_int (List.length (List.nth pou 0));
  print_endline "";

  let st = mat_list "mkb_mat_st" in
  print_int (List.length st);
  print_string " ";
  print_int (List.length (List.nth st 0));
  print_endline ""
