
  (* 横ベクトルの計算 *)
let ( +$ ) u v =
  Array.init (Array.length u) (fun n -> u.(n) +. v.(n))
let ( -$ ) u v =
  Array.init (Array.length u) (fun n -> u.(n) -. v.(n))
let ( *$ ) c v =
  Array.init (Array.length v) (fun n -> c *. v.(n))
    
let print_vector v =
  Array.iter (fun x -> print_string ((string_of_float x) ^ ",")) v;
  print_endline ""

(* 行列の計算 *)
let ( +! ) m1 m2 =
  Array.init (Array.length m1) (fun n -> m1.(n) +$ m2.(n))
let ( -! ) m1 m2 =
  Array.init (Array.length m1) (fun n -> m1.(n) -$ m2.(n))
let ( *< ) c m =
  Array.init (Array.length m) (fun n -> c *$ m.(n))

  
let print_vector v =
  print_endline ("A(" ^ (string_of_int (Array.length v)) ^ ") =");
  Array.iter (fun w -> print_string ((string_of_float w) ^ " ")) v;
  print_endline ""

let print_matrix arr2 =
  print_endline ("A(" ^ (string_of_int (Array.length arr2)) ^ "," ^ (string_of_int (Array.length arr2.(0))) ^ ") = ");
  Array.iter (fun r -> print_vector r) arr2;
  print_endline ""


let kronecker i j = if i = j then 1. else 0.
let canonical_base dim n = Array.init dim (kronecker n)
let diag d_arr =
  let dim = Array.length d_arr in
  Array.init dim (fun k -> d_arr.(k) *$ (canonical_base dim k))

(* テンソル和 *)
let tensor_plus m1 m2 =
  let dim1 = Array.length m1 in
  let dim2 = Array.length m2 in
  let dim = dim1 + dim2 in
  let elem i j =
    if i < dim1 && j < dim1 then m1.(i).(j)
    else if i >= dim1 && j >= dim1 then m2.(i - dim1).(j - dim1)
    else 0.
  in
  let row_vector i = Array.init dim (elem i) in
  Array.init dim row_vector
    
let ( @+ ) m1 m2 = tensor_plus m1 m2

(* transpose : 転置 *)
let trans m =
  let org_row = Array.length m in
  let org_col = Array.length m.(0) in
  let trans_row m n = Array.init org_row (fun k -> m.(k).(n)) in
  Array.init org_col (trans_row m)

let ( @| ) m1 m2 =
  let dim = Array.length m1 in
  Array.init dim (fun k -> Array.append m1.(k) m2.(k))
	
let ( @/ ) m1 m2 = Array.append m1 m2

(* Linear Solution ax = b : with lacaml *)
(* 前進消去法 *)
let fw_elim mat =
  let l = Array.length mat in

  (* j番目の行ベクトルの掃きだし *)
  let fw_elim_row mat_p i j =
    if i >= j then mat_p.(j)
    else
      begin
	let c_ij = mat_p.(j).(i) /. mat_p.(i).(i) in
	mat_p.(j) -$ (c_ij *$ mat_p.(i))
      end
  in

  let rec fw_elim_each mat_pre n =
      if n >= (l-1) then mat_pre
      else
	begin
	  let mat_cur = Array.init l (fw_elim_row mat_pre n) in
	  fw_elim_each mat_cur (n+1)
	end
  in
  fw_elim_each mat 0

(* 後退代入法 *)
let bw_subst mat =
  let l = Array.length mat in
  let bw_subst_base mat_p i j =
    if i = l-1 then mat_p.(i)
    else mat_p.(i) -$ ((mat_p.(i).(j) /. mat_p.(j).(j)) *$ mat_p.(j))
  in

  let bw_subst_row mat_p row i j =
    if j < i then mat_p.(j) -$ ((mat_p.(j).(i) /. row.(i)) *$ row)
    else if j = i then row
    else mat_p.(j)
  in

  let rec bw_subst_mat mat_pre n =
    let row = bw_subst_base mat_pre n (n+1) in
    let mat_cur = Array.init l (bw_subst_row mat_pre row n) in
    if n = 0 then mat_cur else bw_subst_mat mat_cur (n-1)
  in
      
  bw_subst_mat mat (l-1)


(* 連立方程式 : ax = b の解を計算する関数 *)
let solve a b =
  let l = Array.length a in
  let ext_coeff_elem p_a p_b i j = if j < l then p_a.(i).(j) else -. p_b.(i).(0) in
  let ext_coeff_row p_a p_b i = Array.init (l+1) (ext_coeff_elem p_a p_b i) in
  let ext_coeff_mat p_a p_b = Array.init l (ext_coeff_row p_a p_b) in

  let ab = ext_coeff_mat a b in
  let ab_t = fw_elim ab in
  let s_mat = bw_subst ab_t in
  Array.init l (fun n -> [| -. (s_mat.(n).(l) /. s_mat.(n).(n)) |])  

