
open Realext
open Linear
open Diffeq

(* 複合振り子 *)

let l1 = 0.5
let m1 = 1.
let l2 = 0.5
let m2 = 1.

let g = 9.81

(* 平面の一般化座標 : x, y, theta *)

(* 重心G1から見た O,P の座標 *)
let (x1o, y1o) = (-.l1, 0.)
let (x1p, y1p) = (l1, 0.)

(* 重心G2から見た P,A の座標 *)
let (x2p, y2p) = (-.l2, 0.)
let (x2a, y2a) = (l2, 0.)

let acc (q,v) = 
  let sq3 = sin q.(2).(0) in
  let cq3 = cos q.(2).(0) in
  let sq6 = sin q.(5).(0) in
  let cq6 = cos q.(5).(0) in
	
  (* 質量行列 *)
  let m = (diag [| m1; m1; m1 *. (l1 *^ 2) /. 3. |]) @+ (diag [|m2; m2; m2 *. (l2 *^ 2) /. 3.|]) in
  let fq = [|
    [| 1.; 0.; l1 *. sq3; 0.; 0.; 0. |];
    [| 0.; 1.; -. l1 *. cq3; 0.; 0.; 0. |];
    [| -1.; 0.; l1 *. sq3; 1.; 0.; l2 *. sq6 |];
    [| 0.; -1.; -. l1 *. cq3; 0.; 1.; -. l2 *. cq6 |]
  |] in
  let om = Array.make_matrix 4 4 0. in
  let c_mat = (m @| (trans fq)) @/ (fq @| om) in
  (*Linear.print_matrix c_mat; *)
  let qa = [|
    [|0.|];
    [| -.m1*.g |];
    [|0.|];
    [|0.|];
    [| -.m2*.g |];
    [|0.|]
  |] in
  
  let gamma = [|
    [|-.l1 *. (v.(2).(0)*^2) *.cq3|];
    [|-.l1 *. (v.(2).(0)*^2) *. sq3|];
    [|-. l1 *. (v.(2).(0)*^2) *. cq3 -. l2 *. (v.(5).(0)*^2) *. cq6|];
    [|-. l1 *. (v.(2).(0)*^2) *. sq3 -. l2 *. (v.(5).(0)*^2) *. sq6|]
  |] in
  let b = qa @/ gamma in
  (* Linear.print_matrix b; *)
  let a_ext = solve c_mat b in
  Array.sub a_ext 0 6
    
let dump_rigid_pend (q0, v0) t0 dt n_stop =
  let calc_point q =
    let cq6 = cos q.(5).(0) in
    let sq6 = sin q.(5).(0) in
    let xp = q.(3).(0) +. (x2a *. cq6 -. y2a *. sq6) in
    let yp = q.(4).(0) +. (x2a *. sq6 +. y2a *. cq6) in
    (xp, yp)
  in
  
  let rec dump_next_point (q_pre, v_pre) t0 idx =
    let v_cur = v_pre +! (dt *< acc (q_pre, v_pre)) in
    let q_cur = q_pre +! (dt *< v_pre) in
    let (xp, yp) = calc_point q_cur in
    let t_cur = t0 +. (float_of_int (idx+1)) *. dt in
    print_endline ((string_of_float t_cur) ^ " " ^ (string_of_float xp) ^ " " ^ (string_of_float yp));
    if (idx + 1) >= n_stop then ()
    else dump_next_point (q_cur, v_cur) t0 (idx+1)
  in

  let (xp_0, yp_0) = calc_point q0 in
  print_endline ("0. " ^ (string_of_float xp_0) ^ " " ^ (string_of_float yp_0));
  dump_next_point (q0, v0) 0. 0



let () =
  let s10 = -. pi /. 4. in
  let s20 = -. pi /. 2. in
  
  let q0 = [|
    [| l1 *. (cos s10) |];
    [| l1 *. (sin s10) |];
    [| s10 |];
    [| 2. *. l1 *. (cos s10) +. l2 *. (cos s20) |];
    [| 2. *. l1 *. (sin s10) +. l2 *. (sin s20) |];
    [| s20 |]
  |] in
  
  let v0 = Array.make 6 [|0.|] in
  dump_rigid_pend (q0, v0) 0. 0.001 5000
    
	
